"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocumentationService = void 0;
var KlipperExtendedGcodeParser_1 = require("./KlipperExtendedGcodeParser");
var DocumentationService = /** @class */ (function () {
    function DocumentationService(allGcodes) {
        this.allGcodes = allGcodes;
        this.allGcodesById = Object.fromEntries(Object.entries(this.allGcodes)
            .map(function (_a) {
            var command = _a[0], values = _a[1];
            return values.map(function (value) { return [command, value]; });
        })
            .flat()
            .map(function (commandAndValue) { return [commandAndValue[1].id, commandAndValue]; }));
        this.klipperGcodeParser = new KlipperExtendedGcodeParser_1.KlipperExtendedGcodeParser();
    }
    DocumentationService.prototype.findDocsIds = function (searchString) {
        var parts = searchString.toLowerCase().trim().split(/\s+/g).filter(function (part) { return part; });
        if (!parts.length) {
            return [];
        }
        var idLists = Object.entries(this.allGcodes)
            .map(function (_a) {
            var command = _a[0], doc = _a[1];
            var commandLowerCase = command.toLowerCase();
            if (parts.some(function (part) { return commandLowerCase.includes(part); })) {
                return doc.map(function (docItem) { return docItem.id; });
            }
            var ids = doc
                .map(function (docItem) {
                var title = docItem.title.toLowerCase();
                if (parts.some(function (part) { return title.includes(part); })) {
                    return docItem.id;
                }
                var brief = docItem.brief.toLowerCase();
                if (parts.some(function (part) { return brief.includes(part); })) {
                    return docItem.id;
                }
                return null;
            })
                .filter(function (id) { return id; });
            if (!ids.length) {
                return null;
            }
            return ids;
        })
            .filter(function (ids) { return ids; });
        return Array.from(new Set(idLists.flat()));
    };
    DocumentationService.prototype.findDocs = function (searchString) {
        var _this = this;
        return this.findDocsIds(searchString)
            .map(function (id) { return _this.allGcodesById[id]; });
    };
    DocumentationService.prototype.parseParameters = function (line, parsedParameters) {
        if (parsedParameters === void 0) { parsedParameters = {}; }
        if (!line.words) {
            return parsedParameters;
        }
        for (var _i = 0, _a = line.words.slice(1); _i < _a.length; _i++) {
            var _b = _a[_i], tag = _b[0], value = _b[1];
            parsedParameters[tag] = parsedParameters[tag] || [];
            parsedParameters[tag].push(value);
        }
        return parsedParameters;
    };
    DocumentationService.prototype.getSearchResult = function (commandLine, options) {
        var _a, _b, _c;
        if (options === void 0) { options = {}; }
        var fullOptions = {
            maxResultCount: options.maxResultCount === undefined ? 20 : options.maxResultCount,
            include: {
                Marlin: ((_a = options.include) === null || _a === void 0 ? void 0 : _a.Marlin) === undefined ? true : options.include.Marlin,
                RepRap: ((_b = options.include) === null || _b === void 0 ? void 0 : _b.RepRap) === undefined ? true : options.include.RepRap,
                Klipper: ((_c = options.include) === null || _c === void 0 ? void 0 : _c.Klipper) === undefined ? true : options.include.Klipper,
            },
        };
        commandLine = commandLine.trim();
        if (!commandLine || commandLine === "?") {
            return {
                line: commandLine,
                isEmpty: true,
                isSearch: false,
                docItems: [],
                extraResultsCount: 0,
            };
        }
        var docItems;
        var parsedParameters = {};
        var isSearch = commandLine.startsWith('?');
        if (isSearch) {
            docItems = this.findDocs(commandLine.slice(1));
        }
        else {
            var line = this.klipperGcodeParser.parseLine(commandLine);
            var command = line.words.length
                ? line.words[0].join('') : null;
            var docItemsList = command && this.allGcodes[command]
                ? [[command, this.allGcodes[command]]] : [];
            docItems = docItemsList
                .map(function (_a) {
                var command = _a[0], docItems = _a[1];
                return docItems.map(function (docItem) { return [command, docItem]; });
            })
                .flat();
            this.parseParameters(line, parsedParameters);
        }
        return {
            line: commandLine,
            isEmpty: false,
            isSearch: isSearch,
            parsedParameters: parsedParameters,
            docItems: docItems
                .filter(function (_a) {
                var docItem = _a[1];
                return fullOptions.include[docItem.source];
            })
                .slice(0, fullOptions.maxResultCount)
                .map(function (_a) {
                var command = _a[0], docItem = _a[1];
                return ({
                    command: command,
                    docItem: docItem,
                });
            }),
            extraResultsCount: docItems.length > fullOptions.maxResultCount
                ? docItems.length - fullOptions.maxResultCount : 0,
        };
    };
    return DocumentationService;
}());
exports.DocumentationService = DocumentationService;
//# sourceMappingURL=DocumentationService.js.map