"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KlipperExtendedGcodeParser = void 0;
var GcodeParser_1 = require("./GcodeParser");
var KlipperExtendedGcodeParser = /** @class */ (function () {
    function KlipperExtendedGcodeParser() {
        this.gcodeParser = new GcodeParser_1.GcodeParser();
    }
    KlipperExtendedGcodeParser.prototype.parseLine = function (line, options) {
        if (options === void 0) { options = {}; }
        var result = {
            line: line,
            words: [],
        };
        var nameMatch = this.constructor.re.exec(this.stripComments(line));
        if (!nameMatch) {
            return this.gcodeParser.parseLine(line, options);
        }
        var name = nameMatch[1], parametersText = nameMatch[2];
        result.words.push([name, '']);
        var parametersRest = (parametersText || '').trim();
        while (parametersRest) {
            var parameterMatch = this.constructor.reParameter
                .exec(parametersRest);
            if (!parameterMatch) {
                break;
            }
            var parameterText = parameterMatch[0], parameter = parameterMatch[1], value = parameterMatch[2];
            result.words.push([parameter, value]);
            parametersRest = parametersRest.slice(parameterText.length).trim();
        }
        return result;
    };
    // http://linuxcnc.org/docs/html/gcode/overview.html#gcode:comments
    // Comments can be embedded in a line using parentheses () or for the remainder of a lineusing a semi-colon. The semi-colon is not treated as the start of a comment when enclosed in parentheses.
    KlipperExtendedGcodeParser.prototype.stripComments = function (line) {
        return line
            .replace(this.constructor.re1, '')
            .replace(this.constructor.re2, '');
    };
    KlipperExtendedGcodeParser.re = /^([A-Z][A-Z_]+)(\s+.*)?$/im;
    KlipperExtendedGcodeParser.reParameter = /^\s*([A-Z][A-Z0-9_]+)\s*=\s*(\S*)/im;
    KlipperExtendedGcodeParser.re1 = new RegExp(/\s*\([^\)]*\)/g); // Remove anything inside the parentheses
    KlipperExtendedGcodeParser.re2 = new RegExp(/\s*;.*/g); // Remove anything after a semi-colon to the end of the line, including preceding spaces
    return KlipperExtendedGcodeParser;
}());
exports.KlipperExtendedGcodeParser = KlipperExtendedGcodeParser;
//# sourceMappingURL=KlipperExtendedGcodeParser.js.map